package net.simon987.biomassplugin.event;

import net.simon987.biomassplugin.BiomassBlob;
import net.simon987.biomassplugin.WorldUtils;
import net.simon987.server.GameServer;
import net.simon987.server.ServerConfiguration;
import net.simon987.server.event.GameEvent;
import net.simon987.server.event.GameEventListener;
import net.simon987.server.event.WorldUpdateEvent;
import net.simon987.server.game.World;

import java.util.ArrayList;
import java.util.HashMap;


public class WorldUpdateListener implements GameEventListener {

    private HashMap<World, Long> worldWaitMap = new HashMap<>(200);

    private int minBlobCount;
    private int maxBlobCount;
    private int blobYield;
    private int waitTime;
    private int blobThreshold;

    public WorldUpdateListener(ServerConfiguration config) {

        minBlobCount = config.getInt("minBiomassRespawnCount");
        maxBlobCount = config.getInt("maxBiomassRespawnCount");
        waitTime = config.getInt("biomassRespawnTime");
        blobThreshold = config.getInt("biomassRespawnThreshold");
        blobYield = config.getInt("biomass_yield");

    }

    @Override
    public Class getListenedEventType() {
        return WorldUpdateEvent.class;
    }


    @Override
    public void handle(GameEvent event) {

        World world = ((WorldUpdateEvent) event).getWorld();

        //If there is less than the respawn threshold,
        if (world.getGameObjects(BiomassBlob.class).size() < blobThreshold) {

            //Set a timer for respawn_time ticks
            if (!worldWaitMap.containsKey(world) || worldWaitMap.get(world) == 0L) {
                worldWaitMap.put(world, GameServer.INSTANCE.getGameUniverse().getTime() + waitTime);
            } else {

                long waitUntil = worldWaitMap.get(world);

                if (GameServer.INSTANCE.getGameUniverse().getTime() >= waitUntil) {

                    //If the timer was set less than respawn_time ticks ago, respawn the blobs
                    ArrayList<BiomassBlob> newBlobs = WorldUtils.generateBlobs(world, minBlobCount,
                            maxBlobCount, blobYield);
                    world.getGameObjects().addAll(newBlobs);

                    //Set the 'waitUntil' time to 0 to indicate that we are not waiting
                    worldWaitMap.replace(world, 0L);
                }
            }

        }

    }
}